/* CrossDevExample.rexx - Cross development script that compute fx tables,
 *                        compile the fx (from assembly code) and run it
 * $VER: CrossDevExample.rexx 1.0 (08.06.11)  2011 by Philippe 'OffseT' Rimauro <offset@cpcscene.net>
 * Requires ACE 1.0 or later
 *          rexxmathlib.library (http://aminet.net/package/util/rexx/RexxMathLib)
 *          Pasmo Z80 assembler (must be in the path)
 */

OPTIONS RESULTS
PARSE ARG delay

SIGNAL ON BREAK_C

appname = "ACE Rexx"
LF = "0A"x

/* Look for ACE running */
IF ~SHOW('P','ACE.1') THEN DO
    CALL FatalError('ACE CPC Emulator process not found!','Oops!')
    EXIT
END

/* External libraries */
LoadLib("rexxsupport.library")
LoadLib("rexxmathlib.library")

/*
 * Main
 */
ADDRESS "ACE.1"
"VERSION"

SAY "ACE version" RESULT "found." LF

/* Reset ACE */
SAY "Cleaning up everything and preparing the CPC..."
"RESET HARD"
/* Let the firmware boot */
DELAY(50)
/* Prepare the CPC */
"KEYBOARDSTROKE SYMBOL AFTER 256"
"RAWKEYSTROKE KEY 18"
"KEYBOARDSTROKE MEMORY &7FFF"
"RAWKEYSTROKE KEY 18"
SAY "Done." LF

/* Pause ACE while working */
"PAUSE"

/* Compute tables */
SAY "Computing & poking tables into ACE..."
a=X2D(9000)
b=-0.2
n=52
x=0
DO ang=0 TO 359 BY 2
    n=n+b
    valeur=TRUNC((SIN(DEG(ang))*COS(DEG(ang))+1)*n)
    "POKE ADR" a "VAL" valeur
    /*SAY x '->' valeur*/
    IF ang=180 THEN b=-b
    a=a+1
    x=x+1
END
SAY "1st table done:" x "value computed!"

a=X2D(9800)
x=0
DO ang=-90 TO 629 BY 6
    valeur=TRUNC((SIN(DEG(ang))+1)*23+46)
    "POKE ADR" a "VAL" valeur
    /*SAY x '->' valeur*/
    a=a+1
    x=x+1
END
DO ang=90 TO 809 BY 6
    valeur=TRUNC((SIN(DEG(ang))+1)*23)
    "POKE ADR" a "VAL" valeur
    /*SAY x '->' valeur*/
    a=a+1
    x=x+1
END
SAY "2nd table done:" x "value computed!"
SAY "Done." LF

/* Assemble code */
SAY "Assembling source code..."
ADDRESS command "pasmo --amsdos --nocase CrossDevExample.asm T:CrossDevExample.bin"
/* Check for assembling error... */
IF RC~=0 THEN DO
    START
    EXIT
END
SAY "Done." LF

/* Load code into ACE! */
SAY "Loading code into ACE..."
"LOAD FILE T:CrossDevExample.bin"
/*ADDRESS command "Delete T:CrossDevExample.bin QUIET"*/
SAY "Done." LF

/* Start ACE again */
"START"
/* Auto-launch the assembled program */
SAY "Auto-launching the assembled program!"
"KEYBOARDSTROKE CALL &8000"
"RAWKEYSTROKE KEY 18"
SAY "Done." LF

EXIT

/*
 * External lib loader
 */
LoadLib:
    PARSE ARG lib

    IF SHOW("L", lib)=0 THEN
        IF ADDLIB(lib, 0, -30, 0)=0 THEN
            CALL FatalError(lib' not found!*NPlease install it.','Okay!')
    RETURN RC

/*
 * Fatal error
 */
FatalError:
    PARSE ARG message,button

    ADDRESS command 'RequestChoice >NIL: TITLE "'||appname||'" BODY "'||message||'" GADGETS "'||button||'"'
    EXIT
